@@ -10,7 +10,9 @@ use assert_matches::assert_matches;
 
 use crate::Dependencies;
 
-use super::{BuildFlags, BuildInternalClosureError, Config, EnvVariables, Error, Library};
+use super::{
+    BuildFlags, BuildInternalClosureError, Config, EnvVariables, Error, InternalLib, Library,
+};
 
 lazy_static! {
     static ref LOCK: Mutex<()> = Mutex::new(());
@@ -18,11 +20,11 @@ lazy_static! {
 
 fn create_config(path: &str, env: Vec<(&'static str, &'static str)>) -> Config {
     {
-        // PKG_CONFIG_PATH is read by pkg-config so we need to actually change the env
+        // PKG_CONFIG_PATH is read by pkg-config, so we need to actually change the env
         let _l = LOCK.lock();
         env::set_var(
             "PKG_CONFIG_PATH",
-            &env::current_dir().unwrap().join("src").join("tests"),
+            env::current_dir().unwrap().join("src").join("tests"),
         );
     }
 
@@ -77,7 +79,7 @@ fn good() {
     assert_eq!(testdata.version, "4.5.6");
     assert!(libraries.get_by_name("testmore").is_none());
 
-    assert_eq!(libraries.iter().count(), 2);
+    assert_eq!(libraries.iter().len(), 2);
 
     assert_flags(
         flags,
@@ -101,6 +103,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 "#,
     );
 }
@@ -130,9 +135,10 @@ fn toml_pkg_config_err_version(
     let err = toml(path, env_vars).unwrap_err();
     match err {
         Error::PkgConfig(e) => match e {
-            pkg_config::Error::Failure {
+            pkg_config::Error::ProbeFailure {
                 command: cmd,
                 output: _,
+                name: _,
             } => {
                 let s = format!(">= {}\"", expected_version);
                 assert!(cmd.ends_with(&s));
@@ -150,12 +156,18 @@ fn missing_file() {
 
 #[test]
 fn missing_key() {
-    toml_err_invalid("toml-missing-key", "no package.metadata.system-deps");
+    toml_err_invalid(
+        "toml-missing-key",
+        "missing key `package.metadata.system-deps`",
+    );
 }
 
 #[test]
 fn not_table() {
-    toml_err_invalid("toml-not-table", "package.metadata.system-deps not a table");
+    toml_err_invalid(
+        "toml-not-table",
+        "`package.metadata.system-deps` is not a table",
+    );
 }
 
 #[test]
@@ -167,7 +179,7 @@ fn version_missing() {
 fn version_not_string() {
     toml_err_invalid(
         "toml-version-not-string",
-        "metadata.system-deps.testlib: not a string or table",
+        "`package.metadata.system-deps.testlib`: not a string or a table",
     );
 }
 
@@ -209,6 +221,91 @@ fn override_name() {
     assert_eq!(testlib.version, "1.2.3");
 }
 
+#[test]
+fn fallback_names() {
+    let (libraries, flags) = toml("toml-fallback-names", vec![]).unwrap();
+    let testlib = libraries.get_by_name("test_lib").unwrap();
+    assert_eq!(testlib.name, "testlib");
+    assert_eq!(testlib.version, "1.2.3");
+
+    eprintln!();
+    eprintln!("{flags}");
+    assert_flags(
+        flags,
+        r#"cargo:rustc-link-search=native=/usr/lib/
+cargo:rustc-link-search=framework=/usr/lib/
+cargo:rustc-link-lib=test
+cargo:rustc-link-lib=framework=someframework
+cargo:include=/usr/include/testlib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LINK
+"#,
+    );
+}
+
+#[test]
+fn version_fallback_names() {
+    let (libraries, flags) = toml("toml-version-fallback-names", vec![]).unwrap();
+    let testlib = libraries.get_by_name("test_lib").unwrap();
+    assert_eq!(testlib.name, "testlib");
+    assert_eq!(testlib.version, "1.2.3");
+    assert_flags(
+        flags,
+        r#"cargo:rustc-link-search=native=/usr/lib/
+cargo:rustc-link-search=framework=/usr/lib/
+cargo:rustc-link-lib=test
+cargo:rustc-link-lib=framework=someframework
+cargo:include=/usr/include/testlib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LINK
+"#,
+    );
+
+    // Now try with v2 feature enabled.
+    let (libraries, flags) = toml(
+        "toml-version-fallback-names",
+        vec![("CARGO_FEATURE_V2", "")],
+    )
+    .unwrap();
+    let testlib = libraries.get_by_name("test_lib").unwrap();
+    assert_eq!(testlib.name, "testlib-2.0");
+    assert_eq!(testlib.version, "2.0.0");
+
+    assert_flags(
+        flags,
+        r#"cargo:rustc-link-search=native=/usr/lib/
+cargo:rustc-link-lib=test
+cargo:include=/usr/include/testlib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TEST_LIB_LINK
+"#,
+    );
+}
+
 #[test]
 fn feature_versions() {
     let (libraries, _) = toml("toml-feature-versions", vec![]).unwrap();
@@ -245,12 +342,14 @@ fn feature_versions() {
 
 #[test]
 fn override_search_native() {
+    #[cfg(target_os = "windows")]
+    let paths_env = "/custom/path;/other/path";
+    #[cfg(not(target_os = "windows"))]
+    let paths_env = "/custom/path:/other/path";
+
     let (libraries, flags) = toml(
         "toml-good",
-        vec![(
-            "SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE",
-            "/custom/path:/other/path",
-        )],
+        vec![("SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE", paths_env)],
     )
     .unwrap();
     let testlib = libraries.get_by_name("testlib").unwrap();
@@ -282,6 +381,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 "#,
     );
 }
@@ -318,6 +420,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 "#,
     );
 }
@@ -326,17 +431,23 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
 fn override_lib() {
     let (libraries, flags) = toml(
         "toml-good",
-        vec![("SYSTEM_DEPS_TESTLIB_LIB", "overrided-test other-test")],
+        vec![("SYSTEM_DEPS_TESTLIB_LIB", "overridden-test other-test")],
     )
     .unwrap();
     let testlib = libraries.get_by_name("testlib").unwrap();
-    assert_eq!(testlib.libs, vec!["overrided-test", "other-test"]);
+    assert_eq!(
+        testlib.libs,
+        vec!["overridden-test", "other-test"]
+            .into_iter()
+            .map(|name| InternalLib::new(name.to_string(), false))
+            .collect::<Vec<InternalLib>>()
+    );
 
     assert_flags(
         flags,
         r#"cargo:rustc-link-search=native=/usr/lib/
 cargo:rustc-link-search=framework=/usr/lib/
-cargo:rustc-link-lib=overrided-test
+cargo:rustc-link-lib=overridden-test
 cargo:rustc-link-lib=other-test
 cargo:rustc-link-lib=framework=someframework
 cargo:include=/usr/include/testlib
@@ -355,6 +466,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 "#,
     );
 }
@@ -363,18 +477,18 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
 fn override_framework() {
     let (libraries, flags) = toml(
         "toml-good",
-        vec![("SYSTEM_DEPS_TESTLIB_LIB_FRAMEWORK", "overrided-framework")],
+        vec![("SYSTEM_DEPS_TESTLIB_LIB_FRAMEWORK", "overridden-framework")],
     )
     .unwrap();
     let testlib = libraries.get_by_name("testlib").unwrap();
-    assert_eq!(testlib.frameworks, vec!["overrided-framework"]);
+    assert_eq!(testlib.frameworks, vec!["overridden-framework"]);
 
     assert_flags(
         flags,
         r#"cargo:rustc-link-search=native=/usr/lib/
 cargo:rustc-link-search=framework=/usr/lib/
 cargo:rustc-link-lib=test
-cargo:rustc-link-lib=framework=overrided-framework
+cargo:rustc-link-lib=framework=overridden-framework
 cargo:include=/usr/include/testlib
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_INCLUDE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB
@@ -391,6 +505,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 "#,
     );
 }
@@ -427,6 +544,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 "#,
     );
 }
@@ -447,7 +567,7 @@ fn override_unset() {
     let testlib = libraries.get_by_name("testlib").unwrap();
     assert_eq!(testlib.link_paths, Vec::<PathBuf>::new());
     assert_eq!(testlib.framework_paths, Vec::<PathBuf>::new());
-    assert_eq!(testlib.libs, Vec::<String>::new());
+    assert_eq!(testlib.libs, Vec::<InternalLib>::new());
     assert_eq!(testlib.frameworks, Vec::<String>::new());
     assert_eq!(testlib.include_paths, Vec::<PathBuf>::new());
 
@@ -468,6 +588,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 ",
     );
 }
@@ -485,7 +608,10 @@ fn override_no_pkg_config() {
     let testlib = libraries.get_by_name("testlib").unwrap();
     assert_eq!(testlib.link_paths, Vec::<PathBuf>::new());
     assert_eq!(testlib.framework_paths, Vec::<PathBuf>::new());
-    assert_eq!(testlib.libs, vec!["custom-lib"]);
+    assert_eq!(
+        testlib.libs,
+        vec![InternalLib::new("custom-lib".to_string(), false)]
+    );
     assert_eq!(testlib.frameworks, Vec::<String>::new());
     assert_eq!(testlib.include_paths, Vec::<PathBuf>::new());
 
@@ -507,6 +633,9 @@ cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
 cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
 cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
 ",
     );
 }
@@ -540,7 +669,7 @@ fn test_build_internal(
             .probe(lib)
             .unwrap();
         pkg_lib.version = version.to_string();
-        Ok(Library::from_pkg_config(&lib, pkg_lib))
+        Ok(Library::from_pkg_config(lib, pkg_lib))
     });
 
     match config.probe_full() {
@@ -558,7 +687,7 @@ fn build_internal_always() {
     )
     .unwrap();
 
-    assert_eq!(called, true);
+    assert!(called);
     assert!(libraries.get_by_name("testlib").is_some());
 }
 
@@ -572,13 +701,13 @@ fn build_internal_auto_not_called() {
     )
     .unwrap();
 
-    assert_eq!(called, false);
+    assert!(!called);
     assert!(libraries.get_by_name("testlib").is_some());
 }
 
 #[test]
 fn build_internal_auto_called() {
-    // Version 5 is not available so we should try building
+    // Version 5 is not available, so we should try building
     let (libraries, called) = test_build_internal(
         "toml-feature-versions",
         vec![
@@ -589,13 +718,13 @@ fn build_internal_auto_called() {
     )
     .unwrap();
 
-    assert_eq!(called, true);
+    assert!(called);
     assert!(libraries.get_by_name("testdata").is_some());
 }
 
 #[test]
 fn build_internal_auto_never() {
-    // Version 5 is not available but we forbid to build the lib
+    // Version 5 is not available, but we forbid to build the lib
     let (err, called) = test_build_internal(
         "toml-feature-versions",
         vec![
@@ -607,7 +736,7 @@ fn build_internal_auto_never() {
     .unwrap_err();
 
     assert!(matches!(err, Error::PkgConfig(..)));
-    assert_eq!(called, false);
+    assert!(!called);
 }
 
 #[test]
@@ -652,12 +781,12 @@ fn build_internal_wrong_version() {
             .cargo_metadata(false)
             .probe(lib)
             .unwrap();
-        Ok(Library::from_pkg_config(&lib, pkg_lib))
+        Ok(Library::from_pkg_config(lib, pkg_lib))
     });
 
     let err = config.probe_full().unwrap_err();
     assert!(matches!(err, Error::BuildInternalWrongVersion(..)));
-    assert_eq!(called.get(), true);
+    assert!(called.get());
 }
 
 #[test]
@@ -676,11 +805,11 @@ fn build_internal_fail() {
 
     let err = config.probe_full().unwrap_err();
     assert!(matches!(err, Error::BuildInternalClosureError(..)));
-    assert_eq!(called.get(), true);
+    assert!(called.get());
 }
 
 #[test]
-fn build_internal_always_gobal() {
+fn build_internal_always_global() {
     let called = Rc::new(Cell::new((false, false)));
     let called_clone = called.clone();
     let called_clone2 = called.clone();
@@ -694,7 +823,7 @@ fn build_internal_always_gobal() {
                 .probe(lib)
                 .unwrap();
             pkg_lib.version = version.to_string();
-            Ok(Library::from_pkg_config(&lib, pkg_lib))
+            Ok(Library::from_pkg_config(lib, pkg_lib))
         })
         .add_build_internal("testdata", move |lib, version| {
             let (a, _) = called_clone2.get();
@@ -705,7 +834,7 @@ fn build_internal_always_gobal() {
                 .probe(lib)
                 .unwrap();
             pkg_lib.version = version.to_string();
-            Ok(Library::from_pkg_config(&lib, pkg_lib))
+            Ok(Library::from_pkg_config(lib, pkg_lib))
         });
 
     let libraries = config.probe_full().unwrap();
@@ -715,7 +844,7 @@ fn build_internal_always_gobal() {
 }
 
 #[test]
-fn build_internal_gobal_override() {
+fn build_internal_global_override() {
     // Request to build all libs using global var but disable it for a specific one
     let called = Rc::new(Cell::new((false, false)));
     let called_clone = called.clone();
@@ -736,7 +865,7 @@ fn build_internal_gobal_override() {
             .probe(lib)
             .unwrap();
         pkg_lib.version = version.to_string();
-        Ok(Library::from_pkg_config(&lib, pkg_lib))
+        Ok(Library::from_pkg_config(lib, pkg_lib))
     })
     .add_build_internal("testdata", move |lib, version| {
         let (a, _) = called_clone2.get();
@@ -747,7 +876,7 @@ fn build_internal_gobal_override() {
             .probe(lib)
             .unwrap();
         pkg_lib.version = version.to_string();
-        Ok(Library::from_pkg_config(&lib, pkg_lib))
+        Ok(Library::from_pkg_config(lib, pkg_lib))
     });
 
     let libraries = config.probe_full().unwrap();
@@ -765,7 +894,7 @@ fn build_internal_override_name() {
     )
     .unwrap();
 
-    assert_eq!(called, true);
+    assert!(called);
     assert!(libraries.get_by_name("test_lib").is_some());
 }
 
@@ -789,31 +918,28 @@ fn optional() {
 fn aggregate() {
     let (libraries, _) = toml("toml-two-libs", vec![]).unwrap();
 
+    assert_eq!(libraries.all_libs(), vec!["test", "test2"]);
     assert_eq!(
-        libraries.all_libs().collect::<Vec<&str>>(),
-        vec!["test", "test2"]
-    );
-    assert_eq!(
-        libraries.all_link_paths().collect::<Vec<&PathBuf>>(),
+        libraries.all_link_paths(),
         vec![Path::new("/usr/lib"), Path::new("/usr/lib64")]
     );
     assert_eq!(
-        libraries.all_frameworks().collect::<Vec<&str>>(),
+        libraries.all_frameworks(),
         vec!["someframework", "someotherframework"]
     );
     assert_eq!(
-        libraries.all_framework_paths().collect::<Vec<&PathBuf>>(),
+        libraries.all_framework_paths(),
         vec![Path::new("/usr/lib"), Path::new("/usr/lib64")]
     );
     assert_eq!(
-        libraries.all_include_paths().collect::<Vec<&PathBuf>>(),
+        libraries.all_include_paths(),
         vec![
             Path::new("/usr/include/testanotherlib"),
             Path::new("/usr/include/testlib")
         ]
     );
     assert_eq!(
-        libraries.all_defines().collect::<Vec<_>>(),
+        libraries.all_defines(),
         vec![
             ("AWESOME", &None),
             ("BADGER", &Some("yes".into())),
@@ -821,3 +947,212 @@ fn aggregate() {
         ]
     );
 }
+
+#[test]
+fn os_specific() {
+    let (libraries, _) = toml(
+        "toml-os-specific",
+        vec![("TARGET", "x86_64-alpine-linux-musl")],
+    )
+    .unwrap();
+    assert!(libraries.get_by_name("testdata").is_some());
+    assert!(libraries.get_by_name("testlib").is_some());
+    assert!(libraries.get_by_name("testanotherlib").is_some());
+
+    let (libraries, _) = toml("toml-os-specific", vec![("TARGET", "x86_64-apple-darwin")]).unwrap();
+    assert!(libraries.get_by_name("testdata").is_none());
+    assert!(libraries.get_by_name("testlib").is_none());
+    assert!(libraries.get_by_name("testanotherlib").is_some());
+
+    let (libraries, _) = toml(
+        "toml-os-specific",
+        vec![("TARGET", "x86_64-pc-windows-gnu")],
+    )
+    .unwrap();
+    assert!(libraries.get_by_name("testdata").is_none());
+    assert!(libraries.get_by_name("testlib").is_some());
+    assert!(libraries.get_by_name("testanotherlib").is_none());
+}
+
+#[test]
+fn invalid_cfg() {
+    let err = toml(
+        "toml-invalid-cfg",
+        vec![("TARGET", "x86_64-unknown-linux-gnu")],
+    )
+    .unwrap_err();
+
+    assert_matches!(err, Error::UnsupportedCfg(_));
+}
+
+#[test]
+fn static_one_lib() {
+    let (libraries, flags) = toml(
+        "toml-static",
+        vec![("SYSTEM_DEPS_TESTSTATICLIB_LINK", "static")],
+    )
+    .unwrap();
+
+    let testdata = libraries.get_by_name("testdata").unwrap();
+    assert!(!testdata.statik);
+
+    let testlib = libraries.get_by_name("teststaticlib").unwrap();
+    assert!(testlib.statik);
+
+    assert_flags(
+        flags,
+        r#"cargo:rustc-link-search=native=./src/tests/lib/
+cargo:rustc-link-search=framework=./src/tests/lib/
+cargo:rustc-link-lib=static=teststatic
+cargo:rustc-link-lib=framework=someframework
+cargo:include=./src/tests/include/testlib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+"#
+        .to_string()
+        .as_str(),
+    );
+}
+
+#[test]
+fn override_static_no_pkg_config() {
+    let (libraries, flags) = toml(
+        "toml-static",
+        vec![
+            ("SYSTEM_DEPS_TESTSTATICLIB_NO_PKG_CONFIG", "1"),
+            ("SYSTEM_DEPS_TESTSTATICLIB_LIB", "custom-lib"),
+            ("SYSTEM_DEPS_TESTSTATICLIB_LINK", "static"),
+        ],
+    )
+    .unwrap();
+    let testlib = libraries.get_by_name("teststaticlib").unwrap();
+    assert_eq!(testlib.link_paths, Vec::<PathBuf>::new());
+    assert_eq!(testlib.statik, true);
+    assert_eq!(testlib.framework_paths, Vec::<PathBuf>::new());
+    assert_eq!(
+        testlib.libs,
+        vec![InternalLib::new("custom-lib".to_string(), true)]
+    );
+    assert_eq!(testlib.frameworks, Vec::<String>::new());
+    assert_eq!(testlib.include_paths, Vec::<PathBuf>::new());
+
+    assert_flags(
+        flags,
+        r"cargo:rustc-link-lib=static=custom-lib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+",
+    );
+}
+
+#[test]
+fn static_all_libs() {
+    let (libraries, flags) = toml("toml-static", vec![("SYSTEM_DEPS_LINK", "static")]).unwrap();
+
+    let testdata = libraries.get_by_name("testdata").unwrap();
+    assert!(testdata.statik);
+
+    let testlib = libraries.get_by_name("teststaticlib").unwrap();
+    assert!(testlib.statik);
+
+    assert_flags(
+        flags,
+        r#"cargo:rustc-link-search=native=./src/tests/lib/
+cargo:rustc-link-search=framework=./src/tests/lib/
+cargo:rustc-link-lib=static=teststatic
+cargo:rustc-link-lib=framework=someframework
+cargo:include=./src/tests/include/testlib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTSTATICLIB_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+"#,
+    );
+}
+
+#[test]
+fn static_lib_not_available() {
+    let (libraries, flags) = toml("toml-good", vec![("SYSTEM_DEPS_LINK", "static")]).unwrap();
+
+    let testdata = libraries.get_by_name("testdata").unwrap();
+    assert!(testdata.statik);
+
+    // testlib is not available as static library, which is why it is linked dynamically,
+    // as seen below
+    let testlib = libraries.get_by_name("testlib").unwrap();
+    assert!(testlib.statik);
+
+    assert_flags(
+        flags,
+        r#"cargo:rustc-link-search=native=/usr/lib/
+cargo:rustc-link-search=framework=/usr/lib/
+cargo:rustc-link-lib=test
+cargo:rustc-link-lib=framework=someframework
+cargo:include=/usr/include/testlib
+cargo:rerun-if-env-changed=SYSTEM_DEPS_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTDATA_LINK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LIB
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LIB_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_NATIVE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_SEARCH_FRAMEWORK
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_INCLUDE
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_NO_PKG_CONFIG
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_BUILD_INTERNAL
+cargo:rerun-if-env-changed=SYSTEM_DEPS_TESTLIB_LINK
+"#,
+    );
+}
